from urllib.parse import urlparse

from odoo import fields
from odoo.tests import tagged
from odoo.addons.saas_worker.tests.common import SaasWorkerHttpCommon
from odoo.addons.openerp_portal.tests.common import HttpCaseWithUserPortalAndWirePayment
from odoo.addons.openerp_enterprise.tests.test_subscription import SubscriptionCommon, generate_uuid


@tagged('post_install', '-at_install')
class TestSubscriptionSelfBuy(SaasWorkerHttpCommon, HttpCaseWithUserPortalAndWirePayment,
                              SubscriptionCommon):
    quotation_template = 'openerp_enterprise.custom_template_odoo_monthly'

    @classmethod
    def init_master_data(cls):
        super().init_master_data()
        cls.customer_user = cls.env['res.users'].create({
            'login': "enterprise-customer@example.net",
            "password": "enterprise-customer@example.net",
            'partner_id': cls.customer.id,
        })

    def init_data(self):
        self.database_create_date = fields.Datetime.to_string(fields.Datetime.now())
        self.odoo_user_standard = self.env.ref('openerp_enterprise.product_odoo_user_standard')
        self.odoo_monthly_standard = self.env.ref("openerp_enterprise.standard_template_odoo_monthly")

        self.db_uuid = generate_uuid()
        with self._mock_saas_config(from_role="trial"):
            self.send_ping(self.db_uuid, {}, payload_update={
                'nbr_users': 27,
                'nbr_active_users': 12,
                'nbr_share_users': 15,
                'nbr_active_share_users': 5,
            })
        self.db = self.env["openerp.enterprise.database"].search([('db_uuid', '=', self.db_uuid)])

    def test_self_buy_standard_saas(self):
        """Test subscription self buy for a SaaS database"""

        # saas database default to monthly subscription, so adapt the
        # "wire transfer" payment method for it
        # - ensure it's published and it test mode
        # - allow_tokenization (not advertised as supported, but sufficiant
        #      in test to make it as valid payment provider and pay with it)
        # - allow for monthly freq.
        self._setup_payment_transfer_for_test()
        transfer_provider = self.env.ref('payment.payment_provider_transfer')
        transfer_provider.sudo().write({
            'allow_tokenization': True,
            "allow_monthly_payments": True,
        })
        transfer_provider.payment_method_ids.support_tokenization = True

        self.assertEqual(self.db.hosting, "saas")
        self.assertFalse(self.db.subscription_id)
        self_buy_url = f"{self.base_url()}/odoo-enterprise/upgrade-online/{self.db_uuid}"
        self.start_tour(
            self_buy_url, "enterprise_self_buy_standard_plan",
            login=self.customer_user.login
        )

        self.assertTrue(
            self.db.subscription_id,
            "The database should have been linked to a subscription"
        )
        self.assertEqual(self.db.subscription_id.state, 'sent')

    def test_self_buy_standard_cart_update_multitabs_3481828(self):
        """Test ensure that we enforce quotation template on recurring website order (OPW-3481828)

        This test the behavior when user try to buy a self-subscription, then
        1/ Remove item from the cart (browser tab 1)
        2/ Re-set or increase the quantity of that item (browser tab 2)

        We should reject setting a quantity for recurring product on an order
        line that do not exist otherwise we will miss the corresponding
        quotation template (and recurrence) info
        """
        self.assertFalse(self.db.subscription_id)
        self_buy_url = f"{self.base_url()}/odoo-enterprise/upgrade-online/{self.db_uuid}"
        self.authenticate(self.customer_user.login, self.customer_user.login)
        resp = self.url_open(self_buy_url, allow_redirects=False)
        self.assertEqual(resp.status_code, 303, "User should been redirect")
        self.assertEqual(
            urlparse(resp.headers['Location']).path, "/shop/checkout",
            "User should be automatically redirected to checkout"
        )
        self_sub = self.db.subscription_id
        self.assertTrue(self_sub)
        self.assertTrue(self_sub.is_subscription)
        self.assertEqual(self_sub.sale_order_template_id, self.odoo_monthly_standard)
        self.assertEqual(self_sub.origin, 'SaaS Subscription Subscription Customer')
        standard_users_line = self_sub.order_line.filtered(lambda l: l.product_id == self.odoo_user_standard)
        self.assertTrue(standard_users_line)
        self.assertEqual(
            standard_users_line.product_uom_qty, 12.0,
            "Users quantity should match the database active users"
        )

        # browser tab 1: delete standard plan
        self.url_open("/shop/cart/update_json", json={'params': {
            'product_id': self.odoo_user_standard.id,
            'set_qty': 0,
        }})
        # the subscription should not contains any line (so not a subscription anymore)
        # and some fields (origin, ..) should have been resetted
        self.assertEqual(self.db.subscription_id, self_sub)
        self.assertFalse(self_sub.order_line)
        self.assertTrue(self_sub.is_subscription)  # still True because sub plan was kept
        self.assertFalse(self_sub.origin)
        self.assertFalse(self_sub.sale_order_template_id)

        # browser tab 2: re-add standard plan, should autodetect the quotation template
        self.url_open("/shop/cart/update_json", json={'params': {
            'product_id': self.odoo_user_standard.id,
            'set_qty': 13.0,
        }})
        # Check status of the order
        self.assertEqual(self.db.subscription_id, self_sub)
        self.assertTrue(self_sub.is_subscription)
        self.assertIn(
            self.odoo_user_standard, self_sub.order_line.product_id,
            "Standard User should have been added"
        )
        self.assertEqual(
            self.db.subscription_id.sale_order_template_id,
            self.odoo_monthly_standard,
            "The Quotation template should have been recomputed from the recurrence.",
        )

        # User return on its database and click My Subscription again
        # we should keep the same sale order
        resp = self.url_open(self_buy_url, allow_redirects=False)
        self.assertEqual(resp.status_code, 303, "User should been redirect")
        self.assertEqual(
            urlparse(resp.headers['Location']).path, "/shop/checkout",
            "User should be automatically redirected to checkout"
        )
        # Check that previous order has been cancelled
        self.assertTrue(self.db.subscription_id)
        self.assertNotEqual(self.db.subscription_id, self_sub)
        self.assertEqual(self.db.subscription_id.state, 'draft')
        self.assertEqual(self_sub.state, 'cancel')
